local _G = getfenv(0);
local abs = abs;
local gtt = GameTooltip;
local UnitExists = UnitExists;
local modName = "TipTac";

-- Config Data Variables
local cfg;
local TT_DefaultConfig = {
	showUnitTip = true,
	showStatus = true,
	showGuildRank = false,
	nameType = "title",
	showRealm = "show",
	showTarget = "last",
	targetYouText = "<<YOU>>",
	showTargetedBy = true,

	updateFreq = 0.5,

	colSameGuild = "|cffff32ff",
	colRace = "|cffffffff",
	colLevel = "|cffc0c0c0",
	colorNameByClass = false,
	classColoredBorder = false,
	itemQualityBorder = true,

	reactText = false,
	colReactText1 = "|cffc0c0c0",
	colReactText2 = "|cffff0000",
	colReactText3 = "|cffff7f00",
	colReactText4 = "|cffffff00",
	colReactText5 = "|cff00ff00",
	colReactText6 = "|cff25c1eb",
	colReactText7 = "|cff808080",

	reactColoredBackdrop = false,
	reactColoredBorder = false,
	colReactBack1 = { 0.2, 0.2, 0.2, 1 },
	colReactBack2 = { 0.3, 0, 0, 1 },
	colReactBack3 = { 0.3, 0.15, 0, 1 },
	colReactBack4 = { 0.3, 0.3, 0, 1 },
	colReactBack5 = { 0, 0.3, 0.1, 1 },
	colReactBack6 = { 0, 0, 0.5, 1 },
	colReactBack7 = { 0.05, 0.05, 0.05, 1 },

	tipBackdropBG = "Interface\\Buttons\\WHITE8X8",
	tipBackdropEdge = "Interface\\Tooltips\\UI-Tooltip-Border",
	backdropEdgeSize = 14,
	backdropInsets = 2.5,
	tipColor = { 0.1, 0.1, 0.2, 1.0 },
	tipBorderColor = { 0.3, 0.3, 0.4, 1.0 },
	gradientTip = true,
	gradientColor = { 0.8, 0.8, 0.8, 0.2 },

	modifyFonts = false,
	fontFace = "Fonts\\FRIZQT__.TTF",
	fontSize = 12,
	fontFlags = "",

	classification_normal = "%s ",
	classification_elite = "+%s ",
	classification_worldboss = "%s|r (Boss) ",
	classification_rare = "%s|r (Rare) ",
	classification_rareelite = "+%s|r (Rare) ",

	gttScale = 1,

	overrideFade = true,
	preFadeTime = 0.1,
	fadeTime = 0.1,
	hideWorldTips = true,

	hideDefaultBar = true,
	healthBar = true,
	healthBarClassColor = true,
	healthBarText = "value",
	healthBarColor = { 0.3, 0.9, 0.3, 1 },
	manaBar = false,
	manaBarText = "value",
	manaBarColor = { 0.3, 0.55, 0.9, 1 },
	powerBar = false,
	powerBarText = "value",

	barFontFace = "Fonts\\ARIALN.TTF",
	barFontSize = 12,
	barFontFlags = "OUTLINE",
	barHeight = 6,
	barTexture = "Interface\\TargetingFrame\\UI-StatusBar",

	aurasAtBottom = false,
	showBuffs = true,
	selfBuffsOnly = false,
	showDebuffs = true,
	selfDebuffsOnly = false,
	auraSize = 20,
	auraMaxRows = 2,
	showAuraCooldown = true,

	showIcon = true,
	iconAnchor = "TOPLEFT",
	iconSize = 24,

	anchorWorldUnitType = "normal",
	anchorWorldUnitPoint = "BOTTOMRIGHT",
	anchorWorldTipType = "normal",
	anchorWorldTipPoint = "BOTTOMRIGHT",
	anchorFrameUnitType = "normal",
	anchorFrameUnitPoint = "BOTTOMRIGHT",
	anchorFrameTipType = "normal",
	anchorFrameTipPoint = "BOTTOMRIGHT",

	mouseOffsetX = 0,
	mouseOffsetY = 0,

	hideUFTipsInCombat = false,
	hideAllTipsInCombat = false,
};

-- Tips modified by TipTac in appearance and scale (You can add to this list if you want to modify more tips)
local TT_TipsToModify = {
	"GameTooltip",
	"ShoppingTooltip1",
	"ShoppingTooltip2",
	"ShoppingTooltip3",
	"ItemRefTooltip",
	"WorldMapTooltip",
	"AtlasLootTooltip",
	"QuestHelperTooltip",
	"QuestGuru_QuestWatchTooltip",
};

local tipBackdrop = { tile = false, insets = {} };

-- Constants
local TT_LevelMatch = "^"..TOOLTIP_UNIT_LEVEL:gsub("%%s",".+"); -- Was changed to match other localizations properly, used to match: "^"..LEVEL.." .+" -- Az: doesn't actually match the level line on the russian client!
local TT_NotSpecified = "Not specified";
local TT_Reaction = {
	"Tapped",					-- No localized string of this
	FACTION_STANDING_LABEL2,	-- Hostile
	FACTION_STANDING_LABEL3,	-- Unfriendly (Caution)
	FACTION_STANDING_LABEL4,	-- Neutral
	FACTION_STANDING_LABEL5,	-- Friendly
	FACTION_STANDING_LABEL5,	-- Friendly (Exalted)
	DEAD,						-- Dead
};

-- Colors
local CLASS_COLORS = CUSTOM_CLASS_COLORS or RAID_CLASS_COLORS;
local TT_WHITE = "|cffffffff";
local TT_LIGHTGRAY = "|cffc0c0c0";
local TT_ClassColors = {};
for class, color in pairs(CLASS_COLORS) do
	TT_ClassColors[class] = ("|cff%.2x%.2x%.2x"):format(color.r*255,color.g*255,color.b*255);
end

-- Mirror Anchors
local TT_MirrorAnchors = {
	["TOP"] = "BOTTOM",
	["TOPLEFT"] = "TOPRIGHT",
	["TOPRIGHT"] = "TOPLEFT",
	["BOTTOM"] = "TOP",
	["BOTTOMLEFT"] = "BOTTOMRIGHT",
	["BOTTOMRIGHT"] = "BOTTOMLEFT",
	["LEFT"] = "RIGHT",
	["RIGHT"] = "LEFT",
	["CENTER"] = "CENTER",
};

local TT_MirrorAnchorsSmart = {
	["TOPLEFT"] = "BOTTOMRIGHT",
	["TOPRIGHT"] = "BOTTOMLEFT",
	["BOTTOMLEFT"] = "TOPRIGHT",
	["BOTTOMRIGHT"] = "TOPLEFT",
};

-- Data Variables (u = unit, e = extras)
local pLevel, pGuild;
local u, e = {}, {};
local auras = {};
local bars = {};
local extras = {};
local tipIcon;
TipTac = {};

--------------------------------------------------------------------------------------------------------
--                                Global Chat Message Function (Rev 3)                                --
--------------------------------------------------------------------------------------------------------
if (not AZMSG_REV or AZMSG_REV < 3) then
	AZMSG_REV = 3;
	function AzMsg(text)
		DEFAULT_CHAT_FRAME:AddMessage(tostring(text):gsub("|1","|cffffff80"):gsub("|2","|cffffffff"),128/255,192/255,255/255);
	end
end
--------------------------------------------------------------------------------------------------------
--                                       TipTac Anchor Creation                                       --
--------------------------------------------------------------------------------------------------------

local a = CreateFrame("Frame",nil,UIParent);
a:SetWidth(114);
a:SetHeight(24);
a:SetBackdrop({ bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", tile = true, tileSize = 8, edgeSize = 12, insets = { left = 2, right = 2, top = 2, bottom = 2 } });
a:SetBackdropColor(0.1,0.1,0.2,1);
a:SetBackdropBorderColor(0.1,0.1,0.1,1);
a:SetMovable(1);
a:EnableMouse(1);
a:SetToplevel(1);
a:SetClampedToScreen(1);
a:SetPoint("CENTER");
a:Hide();

a.text = a:CreateFontString(nil,"ARTWORK","GameFontHighlight");
a.text:SetText("TipTacAnchor");
a.text:SetPoint("LEFT",6,0);

a.close = CreateFrame("Button",nil,a,"UIPanelCloseButton");
a.close:SetWidth(24);
a.close:SetHeight(24);
a.close:SetPoint("RIGHT");

-- Events
local function OnEvent(self,event,p1,...)
	-- Cursor Update
	if (event == "CURSOR_UPDATE") then
		if (gtt:IsShown()) and (gtt:IsOwned(UIParent)) and (not u.token) then
			local backup = GameTooltipTextLeft1:GetText();
			gtt:Hide();
			GameTooltipTextLeft1:SetText(backup); -- This is a workaround so that gatherer addons can get the name of nodes
		end
	-- Player Login or Player Level Up
	elseif (event == "PLAYER_LOGIN") then
		pLevel = UnitLevel("player");
	elseif (event == "PLAYER_LEVEL_UP") then
		pLevel = p1;
	-- Variables Loaded
	elseif (event == "VARIABLES_LOADED") then
		if (not TipTac_Config) then
			TipTac_Config = {};
		end
		cfg = TipTac_Config;
		setmetatable(cfg,{ __index = TT_DefaultConfig });
		TipTac:ApplySettings();
		TipTac:HookTips();
		if (cfg.left and cfg.top) then
			a:ClearAllPoints();
			a:SetPoint("TOPLEFT",UIParent,"BOTTOMLEFT",cfg.left,cfg.top);
		else
			a:Show();
			-- just set left and top here, in case the player just closes the anchor without moving, so it doesn't keep showing up when they log in
			cfg.left, cfg.top = a:GetLeft(), a:GetTop();
		end
	end
end

a:SetScript("OnMouseDown",function(self) self:StartMoving() end);
a:SetScript("OnMouseUp",function(self) self:StopMovingOrSizing(); cfg.left, cfg.top = self:GetLeft(), self:GetTop(); end);
a:SetScript("OnEvent",OnEvent);

a:RegisterEvent("PLAYER_LOGIN");
a:RegisterEvent("PLAYER_LEVEL_UP");
a:RegisterEvent("VARIABLES_LOADED");

-- Pointer to Anchor so Options can Access it
TipTac.anchor = a;

--------------------------------------------------------------------------------------------------------
--                                           Slash Handling                                           --
--------------------------------------------------------------------------------------------------------
_G["SLASH_"..modName.."1"] = "/tip";
SlashCmdList[modName] = function(cmd)
	-- Extract Parameters
	local param1, param2 = cmd:match("^([^%s]+)%s*(.*)$");
	param1 = (param1 and param1:lower() or cmd:lower());
	-- Options
	if (param1 == "") then
		local loaded, reason = LoadAddOn("TipTacOptions");
		if (loaded) then
			if (TipTacOptions:IsShown()) then
				TipTacOptions:Hide();
			else
				TipTacOptions:Show();
			end
		else
			AzMsg("Could not open TicTac Options: |1"..tostring(reason).."|r. Please make sure the addon is enabled from the character selection screen.");
		end
	-- Show Anchor
	elseif (param1 == "anchor") then
		a:Show();
	-- Invalid or No Command
	else
		UpdateAddOnMemoryUsage();
		AzMsg(format("----- |2%s|r |1%s|r ----- |1%.2f |2kb|r -----",modName,GetAddOnMetadata(modName,"Version"),GetAddOnMemoryUsage(modName)));
		AzMsg("The following |2parameters|r are valid for this addon:");
		AzMsg(" |2anchor|r = Shows the anchor where the tooltip appears");
	end
end
--------------------------------------------------------------------------------------------------------
--                                   Gather Unit Details & Modify Tip                                 --
--------------------------------------------------------------------------------------------------------

-- Get Reaction Index
local function GetUnitReactionIndex(unit)
	-- Deadies
	if (UnitIsDead(unit)) then
		return 7;
	-- Players (Can't rely on UnitPlayerControlled() alone, since it always returns nil on units out of range)
	elseif (UnitIsPlayer(unit) or UnitPlayerControlled(unit)) then
		if (UnitCanAttack(unit,"player")) then
			if (UnitCanAttack("player",unit)) then
				return 2;
			else
				return 3;
			end
		elseif (UnitCanAttack("player",unit)) then
			return 4;
		elseif (UnitIsPVP(unit) and not UnitIsPVPSanctuary(unit) and not UnitIsPVPSanctuary("player")) then
			return 5;
		else
			return 6;
		end
	-- Tapped
	elseif (UnitIsTapped(unit) and not UnitIsTappedByPlayer(unit)) then
		return 1;
	-- Others
	else
		local reaction = (UnitReaction(unit,"player") or 3);
		if (reaction > 5) then
			return 5;
		elseif (reaction < 2) then
			return 2;
		else
			return reaction;
		end
	end
end

-- Returns the correct difficulty color compared to the player
local function GetDifficultyLevelColor(level)
	level = (level - pLevel);
	if (level > 4) then
		return "|cffff2020"; -- red
	elseif (level > 2) then
		return "|cffff8040"; -- orange
	elseif (level >= -2) then
		return "|cffffff00"; -- yellow
	elseif (level >= -GetQuestGreenRange()) then
		return "|cff40c040"; -- green
	else
		return "|cff808080"; -- gray
	end
end

-- Gather Unit Details
local function GatherUnitDetails()
	u.reaction = cfg["colReactText"..u.reactionIndex];
	-- Players
	if (u.isPlayer) then
		-- class
		u.class, u.classFixed = UnitClass(u.token);
		u.class = " "..(TT_ClassColors[u.classFixed] or TT_WHITE)..u.class;
		-- name
		u.name, u.realm = UnitName(u.token);
		u.shownName = (cfg.nameType == "original" and u.originalName or cfg.nameType == "title" and UnitPVPName(u.token) or u.name);
		u.line1 = (cfg.colorNameByClass and (TT_ClassColors[u.classFixed] or TT_WHITE) or u.reaction)..u.shownName;
		if (u.realm) and (u.realm ~= "") and (cfg.showRealm ~= "none") then
			if (cfg.showRealm == "show") then
				u.line1 = u.line1.." - "..u.realm;
			else
				u.line1 = u.line1.." (*)";
			end
		end
		-- dc, afk or dnd
		if (cfg.showStatus) then
			if (not UnitIsConnected(u.token)) then
				u.line1 = u.line1..TT_WHITE.." <DC>";
			elseif (UnitIsAFK(u.token)) then
				u.line1 = u.line1..TT_WHITE.." <AFK>";
			elseif (UnitIsDND(u.token)) then
				u.line1 = u.line1..TT_WHITE.." <DND>";
			end
		end
		-- guild
		u.guild, u.guildRank = GetGuildInfo(u.token);
		if (u.guild) then
			pGuild = GetGuildInfo("player");
			u.guild = (u.guild ~= pGuild and u.reaction or cfg.colSameGuild).."<"..u.guild..">";
			if (cfg.showGuildRank) then
				u.guild = u.guild.." "..TT_LIGHTGRAY..u.guildRank;
			end
		end
		-- race
		u.race = " "..cfg.colRace..(UnitRace(u.token) or "");
	-- NPCs
	else
		-- name
		u.name = UnitName(u.token);
		u.line1 = u.reaction..u.name;
		-- guild
		u.guild = u.title and u.reaction.."<"..u.title..">";
		u.guildRank = nil;
		-- race
		u.race = "";
		-- class
		u.classFixed = nil;
		u.class = UnitCreatureFamily(u.token) or UnitCreatureType(u.token);
		if (not u.class or u.class == TT_NotSpecified) then
			u.class = "Unknown";
		end
		u.class = " "..cfg.colRace..u.class;
	end
	-- Target
	if (cfg.showTarget == "none") then
		u.target = nil;
	else
		u.target = UnitName(u.token.."target");
		if (u.target == "Unknown") and (not UnitExists(u.token.."target")) then
			u.target = nil;
		elseif (u.target) then
			if (UnitIsUnit("player",u.token.."target")) then
				u.target = TT_WHITE..cfg.targetYouText;
			else
				u.targetReaction = cfg["colReactText"..GetUnitReactionIndex(u.token.."target")];
				if (UnitIsPlayer(u.token.."target")) then
					u.target = u.targetReaction.."["..(TT_ClassColors[select(2,UnitClass(u.token.."target"))] or TT_LIGHTGRAY)..u.target..u.targetReaction.."]";
				else
					u.target = u.targetReaction.."["..u.target.."]";
				end
			end
		end
	end
	-- Level + Classification
	u.level = UnitLevel(u.token);
	if (TTT_Cache) and (u.isPlayer) and (u.level == -1) and (TTT_Cache.name == u.name) and (TTT_Cache.c) then -- Using TipTacTalents, determine level of ?? players by adding talent points together
		u.level = (TTT_Cache.a + TTT_Cache.b + TTT_Cache.c + 9);
	end
	u.levelText = (u.level == -1 and "??" or u.level);
	u.classification = UnitClassification(u.token);
	if (cfg["classification_"..u.classification]) then
		u.levelText = cfg["classification_"..u.classification]:format(u.levelText);
	end
	if (UnitCanAttack(u.token,"player") or UnitCanAttack("player",u.token)) then
		u.levelText = GetDifficultyLevelColor(u.level ~= -1 and u.level or 500)..u.levelText;
	else
		u.levelText = cfg.colLevel..u.levelText;
	end
end

-- Add "Targeted By" line
local function AddTargetedBy()
	e.numParty, e.numRaid = GetNumPartyMembers(), GetNumRaidMembers();
	if (e.numParty > 0 or e.numRaid > 0) then
		e.players, e.counter = "", 0;
		for i = 1, (e.numRaid > 0 and e.numRaid or e.numParty) do
			e.unit = (e.numRaid > 0 and "raid"..i or "party"..i);
			if (UnitIsUnit(e.unit.."target",u.token)) and (not UnitIsUnit(e.unit,"player")) then
				e.color = CLASS_COLORS[select(2,UnitClass(e.unit))];
				e.players = ("%s|cff%.2x%.2x%.2x%s|r, "):format(e.players,e.color.r*255,e.color.g*255,e.color.b*255,UnitName(e.unit));
				e.counter = (e.counter + 1);
			end
		end
		if (e.players ~= "") then
			gtt:AddLine("Targeted By (|cffffffff"..e.counter.."|r): "..e.players:sub(1,-5),nil,nil,nil,1);
		end
	end
end

-- Simple Tip Modification
local function ModifyTip()
	-- Unit Name + Target
	if (not u.target) or (cfg.showTarget == "last") then
		GameTooltipTextLeft1:SetText(u.line1);
	elseif (cfg.showTarget == "first") then
		GameTooltipTextLeft1:SetText(u.line1..TT_WHITE.." : |r"..u.target);
	elseif (cfg.showTarget == "second") then
		GameTooltipTextLeft1:SetText(u.line1.."\n  "..u.target);
	end
	-- Generate InfoLine & Add Target Text + Reaction Text
	local infoLine = u.levelText..u.race..u.class;
	if (u.target) and (cfg.showTarget == "last") then
		infoLine = infoLine.."\n|cffffd100Targeting: "..u.target;
	end
	if (cfg.reactText) then
		infoLine = infoLine.."\n"..cfg["colReactText"..u.reactionIndex]..TT_Reaction[u.reactionIndex];
	end
	-- Guild + Level/Race/Class
	-- Az: this doesn't work with mini diablo, which has the format: 1) Mini Diablo 2) Lord of Terror 3) Player's Pet 4) Level 1 Non-combat Pet
	if (u.guild) then
		GameTooltipTextLeft2:SetText(u.guild);
		GameTooltipTextLeft3:SetText(infoLine);
		GameTooltipTextLeft3:SetTextColor(1,1,1);
	else
		GameTooltipTextLeft2:SetText(infoLine);
		GameTooltipTextLeft2:SetTextColor(1,1,1);
	end
end

--------------------------------------------------------------------------------------------------------
--                                        Health & Power Bars                                         --
--------------------------------------------------------------------------------------------------------

-- Make Two StatusBars
for i = 1, 2 do
	local bar = CreateFrame("StatusBar",nil,gtt);
	bar:SetWidth(1);
	bar:SetHeight(6);

	bar.bg = bar:CreateTexture(nil,"BACKGROUND");
	bar.bg:SetTexture(0.3,0.3,0.3,0.6);
	bar.bg:SetAllPoints();

	bar.text = bar:CreateFontString(nil,"ARTWORK");
	bar.text:SetPoint("CENTER");
	bar.text:SetTextColor(1,1,1);

	tinsert(bars,bar);
end

-- Configures the Health and Power Bars
local function SetupHealthAndPowerBar()
	u.powerType = UnitPowerType(u.token);
	-- Visibility
	if (cfg.healthBar) then
		bars[1]:Show();
		if (u.isPlayer) and (cfg.healthBarClassColor) then
			local color = CLASS_COLORS[u.classFixed];
			bars[1]:SetStatusBarColor(color.r,color.g,color.b);
		else
			bars[1]:SetStatusBarColor(unpack(cfg.healthBarColor));
		end
	else
		bars[1]:Hide();
	end
	if (cfg.powerBar and u.powerType ~= 0) or (cfg.manaBar and u.powerType == 0) then
		if (u.powerType == 0) then
			bars[2]:SetStatusBarColor(unpack(cfg.manaBarColor));
		else
			u.powerColor = PowerBarColor[u.powerType] or { r = 0.5, g = 0.5, b = 0.5 };
			bars[2]:SetStatusBarColor(u.powerColor.r,u.powerColor.g,u.powerColor.b);
		end
		bars[2]:Show();
	else
		bars[2]:Hide();
	end
	-- Anchor Frames
	bars[2]:ClearAllPoints();
	bars[1]:ClearAllPoints();
	if (bars[2]:IsShown()) then
		bars[2]:SetPoint("BOTTOMLEFT",8,9);
		bars[2]:SetPoint("BOTTOMRIGHT",-8,9);
		if (bars[1]:IsShown()) then
			bars[1]:SetPoint("BOTTOMLEFT",bars[2],"TOPLEFT",0,4);
			bars[1]:SetPoint("BOTTOMRIGHT",bars[2],"TOPRIGHT",0,4);
		end
	elseif (bars[1]:IsShown()) then
		bars[1]:SetPoint("BOTTOMLEFT",8,9);
		bars[1]:SetPoint("BOTTOMRIGHT",-8,9);
	end
	-- Calculate the space needed for the shown bars
	bars.offset = 0;
	for _, bar in ipairs(bars) do
		if (bar:IsShown()) then
			bars.offset = (bars.offset + cfg.barHeight + 5);
		end
	end
	if (bars.offset == 0) then
		bars.offset = nil;
	end
	-- Hide GTT Status bar, we have our own, which is prettier!
	if (cfg.hideDefaultBar) then
		GameTooltipStatusBar:Hide();
	end
end

-- Format Number Value
local function FormatValue(val)
	if (val < 10000) then
		return val;
	elseif (val < 1000000) then
		return format("%.1fk",val / 1000);
	else
		return format("%.2fm",val / 1000000);
	end
end

-- Format Bar Text
local function FormatBarValues(val,max,type)
	if (type == "none") then
		return "";
	elseif (type == "auto") then
		return (FormatValue(val).." / "..FormatValue(max));
	elseif (type == "deficit") then
		return (val == max and "" or "-"..(max - val));
	elseif (type == "value") or (max == 0) then -- max should never be zero, but if it is, dont let it pass through to the "percent" type, or there will be an error
		return val.." / "..max;
	elseif (type == "percent") then
		return floor(val / max * 100).."%";
	end
end

-- Update Health & Power
local val, max;
local function UpdateHealthAndPowerBar()
	-- Health
	if (bars[1]:IsShown()) then
		val, max = UnitHealth(u.token), UnitHealthMax(u.token);
		bars[1]:SetMinMaxValues(0,max);
		bars[1]:SetValue(val);
		bars[1].text:SetText(FormatBarValues(val,max,cfg.healthBarText));
	end
	-- Power
	if (bars[2]:IsShown()) then
		val, max = UnitMana(u.token), UnitManaMax(u.token);
		bars[2]:SetMinMaxValues(0,max);
		bars[2]:SetValue(val);
		local barText = (u.powerType == 0 and cfg.manaBarText or cfg.powerBarText);
		bars[2].text:SetText(FormatBarValues(val,max,barText));
	end
end

--------------------------------------------------------------------------------------------------------
--                                       Auras - Buffs & Debuffs                                      --
--------------------------------------------------------------------------------------------------------

local function CreateAura()
	local aura = CreateFrame("Frame",nil,gtt);
	aura:SetWidth(cfg.auraSize);
	aura:SetHeight(cfg.auraSize);
	aura.count = aura:CreateFontString(nil,"OVERLAY");
	aura.count:SetPoint("BOTTOMRIGHT",1,0);
	aura.count:SetFont(GameFontNormal:GetFont(),(cfg.auraSize / 2),"OUTLINE");
	aura.icon = aura:CreateTexture(nil,"BACKGROUND");
	aura.icon:SetAllPoints();
	aura.icon:SetTexCoord(0.07,0.93,0.07,0.93);
	aura.cooldown = CreateFrame("Cooldown",nil,aura,"CooldownFrameTemplate");
	aura.cooldown:SetReverse(1);
	aura.cooldown:SetAllPoints();
	aura.cooldown:SetFrameLevel(aura:GetFrameLevel());
	aura.border = aura:CreateTexture(nil,"OVERLAY");
	aura.border:SetPoint("TOPLEFT",-1,1);
	aura.border:SetPoint("BOTTOMRIGHT",1,-1);
	aura.border:SetTexture("Interface\\Buttons\\UI-Debuff-Overlays");
	aura.border:SetTexCoord(0.296875,0.5703125,0,0.515625);
	tinsert(auras,aura);
	return aura;
end

local function SetupAuras()
	local index = 1;
	local filter, aura;
	local maxAuras = floor(gtt:GetWidth() / (cfg.auraSize + 1));
	local _, iconTexture, count, debuffType, duration, timeLeft;
	-- Get Buffs
	if (cfg.showBuffs) then
		filter = "HELPFUL"..(cfg.selfBuffsOnly and "|PLAYER" or "");
		while (true) do
			_, _, iconTexture, count, debuffType, duration, timeLeft = UnitAura(u.token,index,filter);
			if (not iconTexture) then
				break;
			end
			-- Get Aura Frame
			aura = auras[index] or CreateAura();
			-- Anchor It
			aura:ClearAllPoints();
			if (index == 1) then
				if (cfg.aurasAtBottom) then
					aura:SetPoint("TOPLEFT",gtt,"BOTTOMLEFT");
				else
					aura:SetPoint("BOTTOMLEFT",gtt,"TOPLEFT");
				end
			elseif (maxAuras == 1) or (index % maxAuras == 1) then
				if (cfg.aurasAtBottom) then
					aura:SetPoint("TOP",auras[index - maxAuras],"BOTTOM",0,1);
				else
					aura:SetPoint("BOTTOM",auras[index - maxAuras],"TOP",0,1);
				end
			else
				aura:SetPoint("LEFT",auras[index - 1],"RIGHT",1,0);
			end
			-- Cooldown
			if (cfg.showAuraCooldown) and (duration and duration > 0 and timeLeft and timeLeft > 0) then
				aura.cooldown:SetCooldown(timeLeft - duration,duration);
			else
				aura.cooldown:Hide();
			end
			-- Set Texture + Count
			aura.icon:SetTexture(iconTexture);
			aura.count:SetText(count and count > 1 and count or "");
			-- Border
			aura.border:Hide();
			-- Show + Next, Break if exceed max desired rows of auras
			aura:Show();
			index = (index + 1);
			if (index / maxAuras > cfg.auraMaxRows) then
				break;
			end
		end
	end
	-- Get Debuffs
	if (cfg.showDebuffs) and (index / maxAuras <= cfg.auraMaxRows) then
		local buffCount, debuffIndex = (index - 1), 1;
		local lineOffset = floor(buffCount / maxAuras);
		local buffsOnFirstLine = (maxAuras - buffCount % maxAuras);
		filter = "HARMFUL"..(cfg.selfDebuffsOnly and "|PLAYER" or "");
		while (true) do
			_, _, iconTexture, count, debuffType, duration, timeLeft = UnitAura(u.token,debuffIndex,filter);
			if (not iconTexture) then
				break;
			end
			-- Get Aura Frame
			aura = auras[index] or CreateAura();
			-- Anchor It
			aura:ClearAllPoints();
			if (debuffIndex <= buffsOnFirstLine) and (debuffIndex == 1) then
				if (cfg.aurasAtBottom) then
					aura:SetPoint("TOPRIGHT",gtt,"BOTTOMRIGHT",-1,(cfg.auraSize + 1) * lineOffset);
				else
					aura:SetPoint("BOTTOMRIGHT",gtt,"TOPRIGHT",-1,(cfg.auraSize + 1) * lineOffset);
				end
			elseif ((index - 1) % maxAuras == 0) then
				if (cfg.aurasAtBottom) then
					aura:SetPoint("TOPRIGHT",gtt,"BOTTOMRIGHT",0,(cfg.auraSize + 1) * floor(index / maxAuras));
				else
					aura:SetPoint("BOTTOMRIGHT",gtt,"TOPRIGHT",0,(cfg.auraSize + 1) * floor(index / maxAuras));
				end
			else
				aura:SetPoint("RIGHT",auras[index - 1],"LEFT",-1,0);
			end
			-- Cooldown
			if (cfg.showAuraCooldown) and (duration and duration > 0 and timeLeft and timeLeft > 0) then
				aura.cooldown:SetCooldown(timeLeft - duration,duration);
			else
				aura.cooldown:Hide();
			end
			-- Set Texture + Count
			aura.icon:SetTexture(iconTexture);
			aura.count:SetText(count and count > 1 and count or "");
			-- Border
			local color = DebuffTypeColor[debuffType] or DebuffTypeColor["none"];
			aura.border:SetVertexColor(color.r,color.g,color.b);
			aura.border:Show();
			-- Show + Next, Break if exceed max desired rows of aura
			aura:Show();
			index = (index + 1);
			debuffIndex = (debuffIndex + 1);
			if (index / maxAuras > cfg.auraMaxRows) then
				break;
			end
		end
	end
	-- Hide the Unused
	for i = index, #auras do
		auras[i]:Hide();
	end
end

--------------------------------------------------------------------------------------------------------
--                                          GameTooltip Hooks                                         --
--------------------------------------------------------------------------------------------------------

-- USEFUL NOTES :: This is apparently the order at which the GTT construsts the tip
--		GameTooltip_SetDefaultAnchor()
--		Internal_Funtion_Which_Fills_Tip()		-- GTT:GetUnit() becomes valid after this!
--		GTT:Show()								-- Will Resize the tip
--		GTT.OnTooltipSetUnit()
--		Something that resizes the tip, Show() already does this, but it's also done after OnTooltipSetUnit() again. Or maybe it's just another addon's hook doing it. Tested without any addons loaded, and it still resizes after.

-- GTT Control Variables
local gtt_lastUpdate = 0;
local gtt_newHeight;
local gtt_anchorType;
local gtt_anchorPoint;

-- Get The Anchor Position Depending on the Tip Content and Parent Frame -- Do not depend on "u.token" here, as it might not have been cleared yet!
-- Checking "mouseover" here isn't ideal due to actionbars , they sometimes return true because of selfcast.
local function GetAnchorPosition()
	local mouseFocus = GetMouseFocus();
	local isUnit = UnitExists("mouseover") or (mouseFocus and mouseFocus:GetAttribute("unit"));
	local var = "anchor"..(mouseFocus == WorldFrame and "World" or "Frame")..(isUnit and "Unit" or "Tip");
	return cfg[var.."Type"], cfg[var.."Point"];
end

-- Replace GameTooltip_SetDefaultAnchor (For Re-Anchoring)
function GameTooltip_SetDefaultAnchor(tooltip,parent)
	-- Return if no tooltip or parent
	if (not tooltip or not parent) then
		return;
	end
	-- Position Tip to Normal, Mouse or Parent anchor
	gtt_anchorType, gtt_anchorPoint = GetAnchorPosition();
	tooltip:SetOwner(parent,"ANCHOR_NONE");
	if (gtt_anchorType == "mouse") then
		TipTac:AnchorFrameToMouse(tooltip);
	elseif (gtt_anchorType == "parent") then
		tooltip:SetPoint(TT_MirrorAnchorsSmart[gtt_anchorPoint] or TT_MirrorAnchors[gtt_anchorPoint],parent,gtt_anchorPoint);
	else
		tooltip:SetPoint(gtt_anchorPoint,a);
	end
	tooltip.default = 1;
end

-- HOOK: This is the function that determines the GTT color for the first line for a unit
local Real_GameTooltip_UnitColor = GameTooltip_UnitColor;
function GameTooltip_UnitColor(unit,...)
	if (not cfg.showUnitTip) and (cfg.colorNameByClass) and (UnitIsPlayer(unit)) then
		local color = CLASS_COLORS[select(2,UnitClass(unit))];
		return color.r, color.g, color.b;
	else
		return Real_GameTooltip_UnitColor(unit,...);
	end
end

-- HOOK: GTT:Show -- If there are any bar offsets, resize the tip
local gttShow = gtt.Show;
gtt.Show = function(self,...)
local old = self:GetHeight()
	gttShow(self,...);
	if (bars.offset) then
		gtt_newHeight = (gtt:GetHeight() + bars.offset);
		gtt:SetHeight(gtt_newHeight);
		-- Az: Auctioneer/Informant Workaround, Just need a fixed tooltip name for it to work properly
		if (cfg.tempAuctioneerFix) and (_G["LibExtraTip_1_$Revision: 184 $Tooltip1"]) then
			_G["LibExtraTip_1_$Revision: 184 $Tooltip1"]:ClearAllPoints();
		end
	end
end

-- HOOK: GTT:FadeOut -- This allows us to check when the tip is fading out.
local gttFadeOut = gtt.FadeOut;
gtt.FadeOut = function(self,...)
	if (not u.token) or (not cfg.overrideFade) then
		gtt.fadeOut = 2; -- Don't allow the OnUpdate handler to run the fadeout/update code
		gttFadeOut(self,...);
	elseif (cfg.preFadeTime == 0 and cfg.fadeTime == 0) then
		gtt:Hide();
	else
		gtt.fadeOut = 1;
		gtt_lastUpdate = 0;
		if (u.token) and (cfg.showUnitTip) then
			ModifyTip();
		end
	end
end

-- HOOK: GTT OnShow -- This ensures that default anchored world frame tips have the proper color, their internal function seems to set them to a dark blue color
local function GTTHook_OnShow(self,...)
	gtt_anchorType, gtt_anchorPoint = GetAnchorPosition();
	if (gtt_anchorType == "mouse") and (gtt:GetAnchorType() ~= "ANCHOR_CURSOR") then
		TipTac:AnchorFrameToMouse(self);
	end
	if (self:IsOwned(UIParent)) and (not gtt:GetUnit()) then
		self:SetBackdropColor(unpack(cfg.tipColor));
	end
end

-- HOOK: GTT OnUpdate
local function GTTHook_OnUpdate(self,elapsed)
	-- This ensures that mouse anchored world frame tips have the proper color, their internal function seems to set them to a dark blue color
	if (gtt:GetAnchorType() == "ANCHOR_CURSOR") then
		gtt:SetBackdropColor(unpack(cfg.tipColor));
		gtt:SetBackdropBorderColor(unpack(cfg.tipBorderColor));
		return;
	-- Anchor GTT to Mouse
	elseif (gtt_anchorType == "mouse") then
		TipTac:AnchorFrameToMouse(self);
	end

	-- Fadeout / Update Tip if Showing a Unit
	-- Do not allow (fadeOut == 2), as that is only for non overridden fadeouts, but we still need to keep resizing the tip
	if (u.token) and (gtt.fadeOut ~= 2) then
		gtt_lastUpdate = (gtt_lastUpdate + elapsed);
		if (gtt.fadeOut) then
			gtt:Show(); -- Overrides gtt:FadeOut()
			if (gtt_lastUpdate > cfg.fadeTime + cfg.preFadeTime) then
				gtt.fadeOut = nil;
				gtt:Hide();
			elseif (gtt_lastUpdate > cfg.preFadeTime) then
				gtt:SetAlpha(1 - (gtt_lastUpdate - cfg.preFadeTime) / cfg.fadeTime);
			end
		-- This is only really useful for worldframe unit tips, when u.token == "mouseover", they do not call the GTT:FadeOut()
		elseif (not UnitExists(u.token)) then
			gtt:FadeOut();
		elseif (cfg.updateFreq > 0) and (gtt_lastUpdate > cfg.updateFreq) then
			gtt_lastUpdate = 0;
			TipTac:ApplyGeneralAppearance();
		end
	end

	-- Resize the Tooltip if it's size has changed more than 0.1 units
	if (gtt_newHeight) and (abs(gtt:GetHeight() - gtt_newHeight) > 0.1) then
		gtt:SetHeight(gtt_newHeight);
	end
end

-- HOOK: GTT OnTooltipSetUnit
local function GTTHook_OnTooltipSetUnit(self,...)
	if (cfg.hideAllTipsInCombat or cfg.hideUFTipsInCombat and gtt:GetOwner() ~= UIParent) and (UnitAffectingCombat("player")) then
		gtt:Hide();
		return;
	end
	u.token = select(2,gtt:GetUnit());
	-- Concated unit tokens such as "targettarget" cannot be returned as the unit by GTT:GetUnit() and it will return as "mouseover", but the "mouseover" unit is still invalid at this point for those unitframes!
	-- To overcome this problem, we look if the mouse is over a unitframe, and if that unitframe has a .unit value, then use that. Might have to use the unitframe's unit attribute instead?
	-- A mage's mirror images sometimes doesn't return a "mouseover" unit.
	if (not u.token) then
		local mFocus = GetMouseFocus();
		u.token = mFocus and mFocus:GetAttribute("unit");
	end
	-- Sometimes when you move your mouse quicky over units in the worldframe, we can get here without a unit
	if (not u.token) then
		gtt:Hide();
		return;
	end
	-- A "mouseover" unit is better to have as we can then safely say the tip should no longer show when it becomes invalid. Harder to say with a "party2" unit.
	-- This also helps fix the problem that "mouseover" units aren't valid for group members out of range, a bug that has been in WoW since 3.0.2 I think.
	if (UnitIsUnit(u.token,"mouseover")) then
		u.token = "mouseover";
	end
	gtt.fadeOut = nil; -- Az: Sometimes this wasn't getting reset, the fact a cleanup isn't performed now that it was moved to "OnTooltipCleared" is very bad, so this is a fix
	TipTac:ApplyGeneralAppearance(true);
end

-- HOOK: GTT OnTooltipCleared -- This will clean up auras, bars, raid icon and vars for the gtt when we aren't showing a unit
local function GTTHook_OnTooltipCleared(self,...)
	u.token = nil;
	gtt_lastUpdate = 0;
	gtt_newHeight = nil;
	gtt.fadeOut = nil;
	bars.offset = nil;
	for i = 1, #bars do
		bars[i]:Hide();
	end
	for i = 1, #auras do
		auras[i]:Hide();
	end
	if (tipIcon) then
		tipIcon:Hide();
	end
end

-- OnHide Script -- Used to default the background and border color
local function TipHook_OnHide(self,...)
	self:SetBackdropColor(unpack(cfg.tipColor));				-- Default: For most: (0.1,0.1,0.2), World Objects?: (0,0.2,0.35)
	self:SetBackdropBorderColor(unpack(cfg.tipBorderColor));	-- Default: (1,1,1,1)
end

-- OnTooltipSetItem Script -- For item colored border
local function TipHook_OnTooltipSetItem(self,...)
	local item = (cfg.itemQualityBorder and select(2,self:GetItem()));
	if (item) then
		self:SetBackdropBorderColor(GetItemQualityColor(select(3,GetItemInfo(item)) or 0));
	end
end

-- Function to loop through tips to modify and hook
function TipTac:HookTips()
	-- Need to be called as late as possible during load, as we want to try and be the last addon to hook "OnTooltipSetUnit" so we always have a "completed" tip to work on
	gtt:HookScript("OnShow",GTTHook_OnShow);
	gtt:HookScript("OnUpdate",GTTHook_OnUpdate);
	gtt:HookScript("OnTooltipSetUnit",GTTHook_OnTooltipSetUnit);
	gtt:HookScript("OnTooltipCleared",GTTHook_OnTooltipCleared);
	-- HOOK: OnHide & OnTooltipSetItem Scripts
	for _, tip in ipairs(TT_TipsToModify) do
		tip = _G[tip];
		if (type(tip) == "table") and (type(tip.GetObjectType) == "function") and (tip:GetObjectType() == "GameTooltip") then
			tip:HookScript("OnHide",TipHook_OnHide);
			tip:HookScript("OnTooltipSetItem",TipHook_OnTooltipSetItem);
		end
	end
	-- Clear this function as it's not needed anymore
	self.HookTips = nil;
end

--------------------------------------------------------------------------------------------------------
--                                              Settings                                              --
--------------------------------------------------------------------------------------------------------

-- Setup Gradient Tip
local function SetupGradientTip(tip)
	local g = tip.tipTacGradient;
	if (not cfg.gradientTip) then
		if (g) then
			g:Hide();
		end
		return;
	elseif (not g) then
		g = tip:CreateTexture();
		g:SetTexture(1,1,1,1);
--		g:SetTexture("Interface\\Tooltips\\UI-Tooltip-Background");
		tip.tipTacGradient = g;
	end
--	t:SetGradientAlpha("VERTICAL",0,0,0,0,gtt:GetBackdropColor());
	g:SetGradientAlpha("VERTICAL",0,0,0,0,unpack(cfg.gradientColor));
	g:SetPoint("TOPLEFT",cfg.backdropInsets,cfg.backdropInsets * -1);
	g:SetPoint("BOTTOMRIGHT",tip,"TOPRIGHT",cfg.backdropInsets * -1,-36);
	g:Show();
end

-- Apply Settings
function TipTac:ApplySettings()
	-- Hide World Tips (use CURSOR_UPDATE)
	if (cfg.hideWorldTips) then
		a:RegisterEvent("CURSOR_UPDATE");
	else
		a:UnregisterEvent("CURSOR_UPDATE");
	end
	-- Set Backdrop
	tipBackdrop.bgFile = cfg.tipBackdropBG;
	tipBackdrop.edgeFile = cfg.tipBackdropEdge;
	tipBackdrop.edgeSize = cfg.backdropEdgeSize;
	tipBackdrop.insets.left = cfg.backdropInsets;
	tipBackdrop.insets.right = cfg.backdropInsets;
	tipBackdrop.insets.top = cfg.backdropInsets;
	tipBackdrop.insets.bottom = cfg.backdropInsets;
	-- Set Scale, Backdrop, Gradient
	for _, tip in ipairs(TT_TipsToModify) do
		tip = _G[tip];
		if (type(tip) == "table") and (type(tip.GetObjectType) == "function") and (tip:GetObjectType() == "GameTooltip") then
			SetupGradientTip(tip);
			tip:SetScale(cfg.gttScale);
			tip:SetBackdrop(tipBackdrop);
			tip:SetBackdropColor(unpack(cfg.tipColor));
			tip:SetBackdropBorderColor(unpack(cfg.tipBorderColor));
		end
	end
	-- Bar Appearances
	GameTooltipStatusBar:SetStatusBarTexture(cfg.barTexture);
	GameTooltipStatusBar:SetHeight(cfg.barHeight);
	for _, bar in ipairs(bars) do
		bar:SetStatusBarTexture(cfg.barTexture);
		if (not bar:GetStatusBarTexture()) then
			bar:SetStatusBarTexture("Interface\\TargetingFrame\\UI-StatusBar");
		end
		bar:SetHeight(cfg.barHeight);
		bar.text:SetFont(cfg.barFontFace,cfg.barFontSize,cfg.barFontFlags);
	end
	-- If disabled, hide auras, else set their size
	for _, aura in ipairs(auras) do
		if (cfg.showBuffs or cfg.showDebuffs) then
			aura:SetWidth(cfg.auraSize);
			aura:SetHeight(cfg.auraSize);
			aura.count:SetFont("Fonts\\FRIZQT__.TTF",(cfg.auraSize / 2),"OUTLINE");
		else
			aura:Hide();
		end
	end
	-- GameTooltip Font Templates
	if (cfg.modifyFonts) then
		GameTooltipHeaderText:SetFont(cfg.fontFace,cfg.fontSize + 2,cfg.fontFlags);
		GameTooltipText:SetFont(cfg.fontFace,cfg.fontSize,cfg.fontFlags);
		GameTooltipTextSmall:SetFont(cfg.fontFace,cfg.fontSize - 2,cfg.fontFlags);
	end
	-- Raid Icon
	if (cfg.showIcon) and (not tipIcon) then
		tipIcon = gtt:CreateTexture(nil,"BACKGROUND");
	end
	if (tipIcon) then
		if (cfg.showIcon) then
			tipIcon:SetWidth(cfg.iconSize);
			tipIcon:SetHeight(cfg.iconSize);
			tipIcon:ClearAllPoints();
			tipIcon:SetPoint(TT_MirrorAnchors[cfg.iconAnchor],gtt,cfg.iconAnchor);
		else
			tipIcon:Hide();
		end
	end
end

--------------------------------------------------------------------------------------------------------
--                                          TipTac Functions                                          --
--------------------------------------------------------------------------------------------------------

-- Allows other mods to "register" tooltips to be modified by TipTac
function TipTac:AddModifiedTip(tooltip)
	if (tooltip:GetName()) then
		tinsert(TT_TipsToModify,tooltip:GetName());
		tooltip:HookScript("OnHide",TipHook_OnHide);
		tooltip:HookScript("OnTooltipSetItem",TipHook_OnTooltipSetItem);
		self:ApplySettings();
	end
end

-- Anchor any given frame to mouse position
function TipTac:AnchorFrameToMouse(frame)
	local x, y = GetCursorPosition();
	local effScale = frame:GetEffectiveScale();
	frame:ClearAllPoints();
	frame:SetPoint(gtt_anchorPoint,UIParent,"BOTTOMLEFT",(x / effScale + cfg.mouseOffsetX),(y / effScale + cfg.mouseOffsetY));
end

-- Apply TipTac Appearance
local function ApplyTipTacAppearance(first)
	-- Store Original Name
	if (first) and (cfg.nameType == "original") then
		u.originalName = GameTooltipTextLeft1:GetText();
	end
	-- Find NPC Title
	if (first) and (not u.isPlayer) then
		u.title = GameTooltipTextLeft2:GetText();
		if (u.title) and (u.title:find(TT_LevelMatch)) then
			u.title = nil;
		end
	end
	-- Modify the Tip
	GatherUnitDetails();
	ModifyTip();
	if (first) and (cfg.showTargetedBy) then
		AddTargetedBy();
	end
	-- Bars
	if (first) or (UnitPowerType(u.token) ~= u.powerType) then
		SetupHealthAndPowerBar();
	end
	UpdateHealthAndPowerBar();
	-- Remove PVP Line, which makes the tip look a bit bad
	local line;
	for i = 2, gtt:NumLines() do
		line = _G["GameTooltipTextLeft"..i];
		if (line:GetText() == PVP_ENABLED) then
			line:SetText(nil);
			break;
		end
	end
	-- Show & Adjust Height
	gtt:Show();
end

-- Apply General Appearance
function TipTac:ApplyGeneralAppearance(first)
	u.isPlayer = UnitIsPlayer(u.token);
	u.reactionIndex = GetUnitReactionIndex(u.token);
	-- Reaction Backdrop/Border Color
	if (cfg.reactColoredBackdrop) then
		gtt:SetBackdropColor(unpack(cfg["colReactBack"..u.reactionIndex]));
	end
	if (cfg.reactColoredBorder) then
		gtt:SetBackdropBorderColor(unpack(cfg["colReactBack"..u.reactionIndex]));
	-- Class Colored Border
	elseif (first) and (cfg.classColoredBorder) and (u.isPlayer) then
		local color = CLASS_COLORS[select(2,UnitClass(u.token))];
		gtt:SetBackdropBorderColor(color.r,color.g,color.b,1);
	end
	-- Raid Icon
	if (cfg.showIcon) and (tipIcon) then
		local raidIconIndex = GetRaidTargetIndex(u.token);
		if (raidIconIndex) then
			tipIcon:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons");
			SetRaidTargetIconTexture(tipIcon,raidIconIndex);
			tipIcon:Show();
		elseif (UnitIsPVPFreeForAll(u.token)) then
			tipIcon:SetTexture("Interface\\TargetingFrame\\UI-PVP-FFA");
			tipIcon:SetTexCoord(0,0.62,0,0.62);
			tipIcon:Show();
		elseif (UnitIsPVP(u.token)) and (UnitFactionGroup(u.token)) then
			tipIcon:SetTexture("Interface\\TargetingFrame\\UI-PVP-"..UnitFactionGroup(u.token));
			tipIcon:SetTexCoord(0,0.62,0,0.62);
			tipIcon:Show();
		elseif (UnitAffectingCombat(u.token)) then
			tipIcon:SetTexture("Interface\\CharacterFrame\\UI-StateIcon");
			tipIcon:SetTexCoord(0.5,1,0,0.5);
			tipIcon:Show();
		else
			tipIcon:Hide();
		end
	end
	-- TipTac Appearance
	if (cfg.showUnitTip) then
		ApplyTipTacAppearance(first);
	end
	-- Auras - Has to be updated last because it depends on the tips new dimention
	if (cfg.showBuffs or cfg.showDebuffs) then
		SetupAuras();
	end
end