local _G = getfenv(0);
local gtt = GameTooltip;

TTT_Cache = {};
local cache = TTT_Cache;
local points = {};
local uToken, val;
local lastQuery = 0;
local TALENTS_PREFIX = "Talents:|cffffffff ";
local cacheSize = 25;		-- Change cache size here (Default 25)
local throttleTime = 0.0;	-- Set how often TipTacTalents are allowed to query talents (Default 0.5) -- Az: currently testing if setting this to zero isn't really just the best

-- GatherTalents
local function GatherTalents(inspect)
	-- Get points per tree, and set "val" to the tree with most points
	for i = 1, 3 do
		points[i] = select(3,GetTalentTabInfo(i,inspect));
		if (i == 1) or (points[i] > points[val]) then
			val = i;
		end
	end
	-- Customize line output. Set "val" to the desired format
	if (points[val] == 0) then
		val = "No Talents";
	else
		if (not TipTac_Config) or (not TipTac_Config.talentFormat) or  (TipTac_Config.talentFormat == 1) then
			val = GetTalentTabInfo(val,inspect).." ("..points[1].."/"..points[2].."/"..points[3]..")";
		elseif (TipTac_Config.talentFormat == 2) then
			val = GetTalentTabInfo(val,inspect);
		elseif (TipTac_Config.talentFormat == 3) then
			val = points[1].."/"..points[2].."/"..points[3];
		end
	end
	-- Set the cache's last queried data and the tips line output, for inspect, only update if the tip is still showing a unit!
	cache.a, cache.b, cache.c = points[1], points[2], points[3];
	if (not inspect) then
		gtt:AddLine(TALENTS_PREFIX..val);
	elseif (gtt:GetUnit())then
		for i = 2, gtt:NumLines() do
			if ((_G["GameTooltipTextLeft"..i]:GetText() or ""):find("^"..TALENTS_PREFIX)) then
				_G["GameTooltipTextLeft"..i]:SetText(TALENTS_PREFIX..val);
				-- Do not call Show() if the tip is fading out, this only works with TipTac, if TipTacTalents are used solo, it might still bug out the fadeout
				if (not gtt.fadeOut) then
					gtt:Show();
				end
				break;
			end
		end
	end
	-- Organise Cache
	for i = 1, #cache do
		if (cache.name == cache[i].name) then
			tremove(cache,i);
			break;
		end
	end
	if (#cache > cacheSize) then
		tremove(cache,1);
	end
	-- Cache the new entry
	if (cacheSize > 0) then
		tinsert(cache,{ name = cache.name, talents = val });
	end
end

-- Event Frame
local f = CreateFrame("Frame");
f:SetScript("OnEvent",function(self,event,...)
	self:UnregisterEvent("INSPECT_TALENT_READY");
	GatherTalents(1);
--	ClearInspectPlayer();
end);

-- HOOK: OnTooltipSetUnit
gtt:HookScript("OnTooltipSetUnit",
function(self,...)
	if (TipTac_Config) and (TipTac_Config.showTalents == false) then
		return;
	end
	uToken = select(2,self:GetUnit());
	-- Check the UnitFrame unit if this tip is from a concated unit, such as "targettarget".
	if (not uToken) then
		local mFocus = GetMouseFocus();
		if (mFocus) and (mFocus.unit) then
			uToken = mFocus.unit;
		end
	end
	-- Only for players over level 9
	if (UnitIsPlayer(uToken)) and (UnitLevel(uToken) > 9 or UnitLevel(uToken) == -1) then
		cache.name = UnitName(uToken);
		-- Player
		if (UnitIsUnit(uToken,"player")) then
			GatherTalents();
		-- Others
		else
			local allowInspect = (not InspectFrame or not InspectFrame:IsShown()) and (not Examiner or not Examiner:IsShown()) and (CheckInteractDistance(uToken,1) and CanInspect(uToken) and (GetTime() - lastQuery > throttleTime));
			if (allowInspect) then
				f:RegisterEvent("INSPECT_TALENT_READY");
				NotifyInspect(uToken);
				lastQuery = GetTime();
			end
			cache.a, cache.b, cache.c = nil, nil, nil;
			for i = 1, #cache do
				if (cache.name == cache[i].name) then
					self:AddLine(TALENTS_PREFIX..cache[i].talents);
					cache.a, cache.b, cache.c = cache[i].talents:match("(%d+)/(%d+)/(%d+)");
					return;
				end
			end
			if (allowInspect) then
				self:AddLine(TALENTS_PREFIX.."Loading...");
			end
		end
	end
end
);